/* Copyright 1997 Jim McBeath
 *
 * Jim McBeath, January 18, 1997
 */

package jimmc.roots;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Dictionary;
import java.util.Enumeration;

/** Command is a superclass for commands used by CommandInterpreter.
 */
public abstract class Command implements PrintHelp {
	CommandInterpreter ci;

	/** Create a Command object. */
	public Command(CommandInterpreter ci) {
		this.ci = ci;
	}

	/** The name of this command.
	 * Subclass must implement this method.
	 * @return the name of this command
	 */
	public abstract String getName();

	/** The action performed by this command.
	 * @return true if OK
	 */
	public abstract boolean action(String words[]);

	/** Print one line of help about the command. */
	public abstract void printOneLineHelp();

	/** Report an error when not enough arguments. */
	protected void missingArgument(String what) {
		ci.base.println(getName()+": Missing argument for "+what);
	}

	/** Report an error when too many arguments */
	protected void tooManyArguments() {
		ci.base.println(getName()+": Too many arguments");
	}
	protected void println(String s) {
		ci.base.println(s);
	}

	/** Add the default commands */
	public static void AddDefaultCommands(CommandInterpreter ci) {
		ci.addCommand(new CommandHelp(ci));
		ci.addCommand(new CommandLoad(ci));
		ci.addCommand(new CommandPrint(ci));
		ci.addCommand(new CommandQuit(ci));
		ci.addCommand(new CommandSelect(ci));
		ci.addCommand(new CommandShow(ci));
	}
}

/** The load command loads a database file. */
class CommandLoad extends Command {
	public CommandLoad(CommandInterpreter ci) {
		super(ci);
	}
	public String getName() {
		return "load";
	}
	public boolean action(String words[]) {
		if (words.length==1) {
			missingArgument("file name");
			return false;
		}
		if (words.length>2) {
			tooManyArguments();
			return false;
		}
		ci.base.readNcvFile(words[1]);
		return true;
	}
	public void printOneLineHelp() {
		println("load <filename> -- load a file into the database");
	}
}

/** The quit command causes the CommandInterpreter to finish. */
class CommandQuit extends Command {
	public CommandQuit(CommandInterpreter ci) {
		super(ci);
	}
	public String getName() {
		return "quit";
	}
	public boolean action(String words[]) {
		if (words.length>1) {
			tooManyArguments();
			return false;
		}
		ci.runMe = false;
		return true;
	}
	public void printOneLineHelp() {
		println("quit -- quit the program");
	}
}

/** Print a record. */
class CommandPrint extends Command {
	public CommandPrint(CommandInterpreter ci) {
		super(ci);
	}
	public String getName() {
		return "print";
	}
	public boolean printAction(PrintOptions po, String words[]) {
		boolean recordFormatSpecified = false;
		String outputFileName = null;
		int i=1;	/* index of first arg */
		while (i<words.length) {	/* look for options */
			if (words[i].equals("-f")) {
				PrintFormat f = ci.findPrintFormat(words[i+1]);
				if (f==null)
println("Can't find print format '"+words[i+1]+"'");
				if (f!=null)
					po.reportFormat = f;
				i += 2;
				continue;
			}
			if (words[i].equals("-k")) {
				po.includeKeys = true;
				i++;
				continue;
			}
			if (words[i].equals("-o")) {
				outputFileName = words[i+1];
				i += 2;
				continue;
			}
			if (words[i].equals("-r")) {
				RecordFormat f = ci.findRecordFormat(words[i+1]);
				if (f==null)
println("Can't find record format '"+words[i+1]+"'");
				if (f!=null) {
					po.recordFormat = f;
					recordFormatSpecified = true;
				}
				i += 2;
				continue;
			}
			if (words[i].startsWith("-")) {
				ci.base.println("Unknown option '"+
						words[i]+"'");
				return false;
			}
			break;	/* must be the first part of the selection */
		}
		if (!recordFormatSpecified) {
			RecordFormat rf =
				po.reportFormat.getDefaultRecordFormat();
			if (rf!=null)
				po.recordFormat = rf;
		}
		if (outputFileName!=null) {
			PrintWriter ps;
			FileOutputStream os;
			try {
				os = new FileOutputStream(outputFileName);
			} catch (IOException e) {
				ci.base.println("Can't open output file '"+
					outputFileName+"'");
				return false;
			}
			po.out = new PrintWriter(os);
		}
		ci.base.printRecords(po,words,i,words.length-i);
		return true;
	}
	public boolean action(String words[]) {
		return printAction(
			(PrintOptions)ci.base.defaultPrintOptions.clone(),
			words);
	}
	public void printOneLineHelp() {
		println("print [-f reportFormat] [-k] [-o outfile] [-r recordFormat] [selection] -- print selected records");
	}
}

/** The select command changes the current record. */
class CommandSelect extends Command {
	public CommandSelect(CommandInterpreter ci) {
		super(ci);
	}
	public String getName() {
		return "select";
	}
	public boolean action(String words[]) {
		if (words.length==1) {
			missingArgument("new current item");
			return false;
		}
		ci.base.select(words,1,words.length-1);
		return true;
	}
	public void printOneLineHelp() {
		println("select [selection] -- make specified selection the current selection");
	}
}

/** The show command is just like the print command except that its
 * default format is different. */
class CommandShow extends CommandPrint {
	public CommandShow(CommandInterpreter ci) {
		super(ci);
	}
	public String getName() {
		return "show";
	}
	public boolean action(String words[]) {
		return printAction(
			(PrintOptions)ci.base.defaultShowOptions.clone(),
			words);
	}
	public void printOneLineHelp() {
		println("show -- just like print, except different default format");
	}
}

/** Give help on what command are available. */
class CommandHelp extends Command {
	public CommandHelp(CommandInterpreter ci) {
		super(ci);
	}
	public String getName() {
		return "help";
	}
	public boolean action(String words[]) {
		Command c;
		if (words.length<=1)
			helpSummary();
		else if (words[1].equals("commands"))
			helpCommands();
		else if (words[1].equals("recordFormats"))
			helpRecordFormats();
		else if (words[1].equals("reportFormats"))
			helpReportFormats();
		else if (words[1].equals("selection"))
			helpSelection();
		else if ((c=(Command)(ci.commandDictionary.get(words[1])))!=null) {
			c.printOneLineHelp();
				/* TBD - print detailed help */
		} else
			return false;
		return true;
	}
	/** Give a help summary. */
	private void helpSummary() {
		println("Type 'help help' for more details on using help.");
		println("Type 'help xxx' for help on command xxx.");
		println("Or substitute any of the following for xxx:");
		println("commands -- list available comamnds");
		println("recordFormats -- list available record formats");
		println("reportFormats -- list available report formats");
		println("selection -- describe how to make selections");
	}
	private void helpCommands() {
		println("Available commands:");
		printDictionaryOneLineHelp(ci.commandDictionary);
	}
	private void helpRecordFormats() {
		println("Available record formats:");
		printDictionaryOneLineHelp(ci.recordFormatDictionary);
	}
	private void helpReportFormats() {
		println("Available report formats:");
		printDictionaryOneLineHelp(ci.printFormatDictionary);
	}
	private void printDictionaryOneLineHelp(Dictionary d) {
		String sa[] = new String[d.size()];
		Enumeration e = d.keys();
		int n = 0;
		while (e.hasMoreElements())
			sa[n++] = (String)e.nextElement();
		Sort.sortStrings(sa);	/* sort command in alpha order */
		for (n=0; n<sa.length; n++) {
			PrintHelp h = (PrintHelp)d.get(sa[n]);
			h.printOneLineHelp();
		}
	}
	private void helpSelection() {
		println("Selections can be absolute or relative.");
		println("Absolute selections refer to a record key;");
		println("relative selections refer to another selection.");
		println("");
		println("If no record keys are specified in the selection,");
		println("then it is assumed to be relative to the current");
		println("selection.");
		println("");
		println("Relative selections are specified as follows:");
		println("  <relation> of <selection>");
		println("where <selection> is a relative or absolute selection,");
		println("and <relation> is one of the following keywords:");
		println("  mother -- mother (I)");
		println("  father -- father (I)");
		println("  parents -- mother and father (I)");
		println("  parentunion -- parents (U)");
		println("  punion - same as parentunion");
		println("  marriage - marriage (U)");
		println("  union -- same as marriage");
		println("  husband -- husband (I)");
		println("  wife -- wife (I)");
		println("  children -- children (I)");
		println("  child -- same as children");
		println("Items marked with (I) return the record of an individual;");
		println("items marked with (U) return the record of a union.");
		println("A selection which is relative to a set of records");
		println("results in the union of that relation of each record.");
		println("A selection with multiple matches gives all matches.");
		println("For example, if a man was married twice, 'wife'");
		println("will return a selection with both wives.");
		println("In all such cases, appending a number (e.g. wife0)");
		println("will return only the specified one (0-based).");
		println("");
		println("The 'recurse' keyword can be prepended to a selection");
		println("to cause the selection to be applied repeatedly");
		println("until there is no more new data.");
		println("'recurseN' where N>=1 will recurse for only the");
		println("specified number of times.  recurse0 is the same as");
		println("recurse, and recurse1 is a no-op.");
		println("");
		println("The 'plus' keyword can be prepended to a selection");
		println("in order to add it to the base selection.");
		println("For example, 'plus mother of 3001' will select");
		println("both 3001 and 3001's mother.");
	}
	public void printOneLineHelp() {
		println("help [keyword] -- give help about commands and usage");
	}
}

/* end */
