/* DatabaseModel.java
 *
 * Jim McBeath, June 7, 1998
 */

package jimmc.roots;

import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;

/** The model for the database.
 */
public class DatabaseModel {
	/** Our RootsBase. */
	protected RootsBase rootsBase;
	/** Our command interpreter. */
	protected CommandInterpreter interp;
	/** Our location. */
	protected String path;
	/** Our log stream. */
	protected PrintWriter logWriter = new PrintWriter(System.out);

	/** Create a model for the specified database file, with the
	 * log output specified. */
	public DatabaseModel(String filename, PrintWriter logStream) {
		setLogWriter(logStream);
		setupBase(filename);
	}

	/** Create a model for the database in the specified file. */
	public DatabaseModel(String filename) {
		setupBase(filename);
	}

	/** Set up the RootsBase */
	protected void setupBase(String filename) {
		rootsBase = new RootsBase();
		interp = new CommandInterpreter(rootsBase);
		path = filename;

		/* TBD - do this delayed, in a separate thread? */
		log("Loading database...\n");
		doCommand(logWriter,"load "+filename);
	}

	/** Get the path to the database. */
	public String getPath() {
		return path;
	}

	/** Get the base database this model is attached to. */
	public RootsBase getBase() {
		return rootsBase;
	}

	/** Get the number of records in this database. */
	public int getRecordCount() {
		return rootsBase.getRecordCount();
	}

	/** Select one or multiple records. */
	public void select(String txt) {
		interp.base.clearSelection();
		log(doCommand("select "+txt));
	}

	/** Get the current selection. */
	public String[] getSelectedKeys() {
		return interp.base.getSelectedKeys();
	}

	/** Get a list of summary strings for the specified keys. */
	public String[] getKeySummaries(String[] keys) {
		String[] summaries = new String[keys.length];
		for (int i=0; i<keys.length; i++)
			summaries[i] = doCommand(
				"print -k -f list -r oneLine "+keys[i]);
		return summaries;
	}

	/** Get a model for a single record. */
	public SingleModel getSingleModel(String key) {
		/* TBD - keep a hashtable of models so we only ever have
		 * one model for a record.
		 */
		return new SingleModel(this,key);
	}

	/** Execute a command, return the results as a string. */
	protected String doCommand(String cmd) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		PrintWriter pw = new PrintWriter(baos);
		doCommand(pw,cmd);
		return baos.toString();
	}

	/** Execute a command, print results to the specified stream. */
	protected void doCommand(PrintWriter pw, String cmd) {
		PrintWriter oldp = interp.base.defaultPrintOptions.out;
		PrintWriter olds = interp.base.defaultShowOptions.out;
		interp.base.defaultPrintOptions.out = pw;
		interp.base.defaultShowOptions.out = pw;
		//TBD - show command and output somewhere?
//pw.println(cmd);
		//commandField.setText(cmd);
		interp.doCommand(cmd);
		//outputTextField.setText(baos.toString());
		interp.base.defaultPrintOptions.out = oldp;
		interp.base.defaultShowOptions.out = olds;
	}

	/** Set the writer to which we log our output. */
	public void setLogWriter(PrintWriter pw) {
		if (pw==null)
			logWriter = new PrintWriter(System.out);
		else
			logWriter = pw;
	}

	/** Log the string to the output. */
	protected void log(String s) {
		logWriter.print(s);
		logWriter.flush();
	}
}

/* end */
