/* ListView.java
 *
 * Jim McBeath, June 7, 1998
 */

package jimmc.roots;

import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import java.awt.Component;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.GridBagConstraints;

/** ListView provides a view into a list of records.
 */
public class ListView extends RootsFrame implements TextPopupListener {
	/** Our database. */
	protected DatabaseModel databaseModel;
	/** Our model. */
	protected ListModel model;
	/** Where we display the count of items in our list. */
	protected JLabel countField;
	/** Where we display our more verbose output. */
	protected ScrolledText outputField;
	/** The output format choice. */
	protected ComboBoxAction outputChoiceField;
	/** The include-keys toggle. */
	protected JCheckBox includeKeysField;

	/** Create a new view for a record. */
	public ListView(DatabaseModel db, String[] keys) {
		super("List");	/* TBD - better name */
		this.databaseModel = db;
		setJMenuBar(createMenuBar());
		initForm();
		addWindowListener();
		setModel(new ListModel(databaseModel,keys));
		pack();
	}

	/** Set the model we are viewing. */
	public void setModel(ListModel m) {
		model = m;
		countField.setText(Integer.toString(m.getCount()));
		setOutputText();
		validate();
	}

	/** Create our menu bar. */
	protected JMenuBar createMenuBar() {
		JMenuBar mb = new JMenuBar();
		mb.add(createFileMenu());
		//JMenu helpMenu = createHelpMenu();
		//mb.add(helpMenu);
		//mb.setHelpMenu(helpMenu);
		return mb;
		/* TBD - add a menu command to save the large text area
		 * to a file. */
	}

	/** Create our File menu. */
	protected JMenu createFileMenu() {
		JMenu m = new JMenu("File");
		MenuAction mi;
		mi = new MenuAction("Save Output As...") {
			public void action() {
				processSaveOutputAs();
			}
		};
		m.add(mi);
		mi = new MenuAction("Show Display Command") {
			public void action() {
				processShowDisplayCommand();
			}
		};
		m.add(mi);
		mi = new MenuAction("Close") {
			public void action() {
				processClose();
			}
		};
		m.add(mi);
		return m;
	}

	/** Create the body of our form. */
	protected void initForm() {
		GridBagger gb = new GridBagger(getContentPane());
		gb.gbc.fill = GridBagConstraints.HORIZONTAL;
		gb.gbc.anchor = GridBagConstraints.WEST;
		gb.add(createInfoArea());
		gb.nextRow();
		gb.gbc.weightx = 1;
		gb.gbc.weighty = 1;
		gb.gbc.fill = GridBagConstraints.BOTH;
		gb.add(createOutputArea());
		/* TBD - add a status line? */
	}
	
	/** Create the info box at the top of the form. */
	protected Component createInfoArea() {
		JPanel p = new JPanel();
		p.setBorder(new TitledBorder(new EtchedBorder(),"List"));
		GridBagger gb = new GridBagger(p);
		gb.gbc.anchor = GridBagConstraints.WEST;
		gb.add(new JLabel("Count:"));
		gb.gbc.fill = GridBagConstraints.HORIZONTAL;
		gb.gbc.weightx = 1;
		gb.add(countField=new JLabel());
		return p;
	}

	/** Create the output area. */
	protected Component createOutputArea() {
		JPanel p = new JPanel();
		p.setBorder(new TitledBorder(new EtchedBorder(),"Output"));
		GridBagger gb = new GridBagger(p);
		gb.add(new JLabel("Display:"));
		outputChoiceField = new ComboBoxAction() {
			public void action() {
				setOutputText();
			}
		};
		outputChoiceField.addItem("list");
		outputChoiceField.addItem("gedcom");
		outputChoiceField.addItem("ncv");
		gb.add(outputChoiceField);
		includeKeysField = new CheckBoxAction("Include Keys") {
			public void action() {
				setOutputText();
			}
		};
		gb.add(includeKeysField);
		gb.nextRow();
		gb.gbc.gridwidth = 4;
		gb.gbc.weightx = 1;
		gb.gbc.weighty = 1;
		gb.gbc.fill = GridBagConstraints.BOTH;
		outputField = new ScrolledTextWithPopup(500,300) {
			public TextPopup createTextPopup() {
				return new ListViewTextPopup(
					ListView.this,
					this);
			}
			public void processSelect(String key) {
				ListView.this.processSelect(key);
			}
		};
		gb.add(outputField);
		return p;
	}

	/** Process the File->SaveOutputAs command. */
	protected void processSaveOutputAs() {
		String text = outputField.getText();
		if (text.length()==0) {
			errorDialog("No text to save.");
			return;
		}
		String filename = fileDialog("Save to file:");
		if (filename==null)
			return;
		saveTextToFile(text,filename);
	}

	/** Process the Show Display Command command */
	protected void processShowDisplayCommand() {
		boolean includeKeys = includeKeysField.isSelected();
		String outputFormat =
			(String)outputChoiceField.getSelectedItem();
		String text = model.getDisplayCommand(outputFormat,includeKeys);
		infoDialog(text);
	}

	/** Close the window. */
	protected void processClose() {
		/* TBD - notify the model. */
		super.processClose();
	}

	/** Select the specified key. */
	public void processSelect(String key) {
		/* Open up a new SingleView for the selected item. */
		(new SingleView(databaseModel,key)).show();
	}

	/** Set the output field. */
	protected void setOutputText() {
		if (includeKeysField==null)
			return;	/* this happens during init */
		boolean includeKeys = includeKeysField.isSelected();
		String outputFormat =
			(String)outputChoiceField.getSelectedItem();
		String text = model.getOutputText(outputFormat,includeKeys);
		outputField.setText(text);
	}
}

class ListViewTextPopup extends TextPopup {
	/** Our ListView. */
	protected ListView listView;

	/** Create the popup. */
	ListViewTextPopup(ListView lv, TextPopupListener listener) {
		super(listener);
	}

	/** Add our menu items. */
	protected void addMenuItems() {
		add(selectMenu=new MenuPush("SingleView") {
			public void action() {
				w.processSelect(id);
			}
		});
	}
}

/* end */
