/* Copyright 1997 Jim McBeath
 *
 * Jim McBeath, January 12, 1997
 */

package jimmc.roots;

import jimmc.treepar.Layout;

import java.io.*;

/** RootsPerson represents one individual.
 */

public class RootsPerson extends RootsRecord {
	String fullName;	/* cached version of formatted full name */
	String lastName;
	String adoptedLastName;
	String firstName;
	String middleName;
	String nickName;
	String prefixName;	/* e.g. Dr., Lord, King, Sir, etc. */
	String suffixName;	/* e.g. Jr., Sr., Esq., R.N., etc. */
	String sex;		/* "M" or "F"; null if unknown */
	RootsEvent birth;
	RootsEvent death;
	RootsEvent burial;
	String parentsString;	/* gets converted into parents var */
	String spousesString;	/* gets converted into spouses var */
	RootsUnion parents;	/* reference to parents */
	RootsUnion spouses[];	/* may have married more than once */
		/* The term "spouse" is used loosely, and includes any
		 * partners with whom this person had a child, whether
		 * married or not. */
	String addressString;
	RootsAddress address;

	public RootsPerson(RootsBase base) {
		super(base);
		birth = new RootsEvent();
		death = new RootsEvent();
		burial = new RootsEvent();
	}

	public static RootsPerson ParseNcvLines(RootsBase base, String lines[]){
		RootsPerson r = new RootsPerson(base);
		int n = 0;
		while (n<lines.length) {
			Ncv nv = new Ncv(lines,n);
			n += nv.numLines;
			r.addNcv(nv);
		}
		return r;
	}
	/* Returns true if we recognized the name and added the value */
	protected boolean addNcv(Ncv nv) {
		if (super.addNcv(nv))
			return true;
		else if (nv.name.equals("LN"))
			lastName = nv.value;
		else if (nv.name.equals("LN.A"))
			adoptedLastName = nv.value;
		else if (nv.name.equals("PN"))
			prefixName = nv.value;
		else if (nv.name.equals("FN"))
			firstName = nv.value;
		else if (nv.name.equals("MN"))
			middleName = nv.value;
		else if (nv.name.equals("NN"))
			nickName = nv.value;
		else if (nv.name.equals("SN"))
			suffixName = nv.value;
		else if (nv.name.equals("SX"))
			sex = nv.value;
		else if (nv.name.equals("B"))
			birth.date = nv.value;
		else if (nv.name.equals("BP"))
			birth.place = nv.value;
		else if (nv.name.equals("D"))
			death.date = nv.value;
		else if (nv.name.equals("DP"))
			death.place = nv.value;
		else if (nv.name.equals("BUR"))
			burial.place = nv.value;
		else if (nv.name.equals("P"))
			parentsString = nv.value;
		else if (nv.name.equals("S"))
			spousesString = nv.value;
		else if (nv.name.equals("ADDR"))
			addressString = nv.value;
		else
			return false;	/* not recognized */
		return true;		/* OK */
	}

	/* Fix up the links in a record that point to another record */
	protected void link() {
		RootsRecord r;
		RootsUnion ua[];
		super.link();
		if ((r=base.findRecord(parentsString))!=null)
			parents = (RootsUnion)r;
		if ((r=base.findRecord(addressString))!=null)
			address = (RootsAddress)r;
		if ((ua=RootsUnion.FindRecordList(base,spousesString))!=null)
			spouses = ua;
	}

	/* Parse the dates in this record */
	protected void parseDates() {
		parseDate(birth,"birth");
		parseDate(death,"death");
		parseDate(burial,"burial");
	}

	/* Find a list of people */
	public static RootsPerson[] FindRecordList(RootsBase base, String s) {
		RootsRecord ra[];
		ra = base.findRecordList(s);
		if (ra==null)
			return null;
		RootsPerson pa[] = new RootsPerson[ra.length];
		for (int i=0; i<ra.length; i++) {
			pa[i] = (RootsPerson)(ra[i]);
		}
		return pa;
	}

	/** Format the key line for a Gedcom record. */
	protected void formatGedcomKey(StringBuffer sb, int level) {
		formatGedcomKey(sb,level,"INDI");
	}
	/** Format the body of a Gedcom record */
	protected void formatGedcomBody(StringBuffer sb, int level) {
		sb.append(level).append(" NAME ");
		if (firstName!=null)
			sb.append(firstName).append(" ");
		if (middleName!=null)
			sb.append(middleName).append(" ");
		if (nickName!=null)
			sb.append("\"").append(nickName).append("\" ");
		if (lastName!=null)
			sb.append("/").append(lastName).append("/");
		sb.append("\n");
		/* Prefix and Suffix names are subordinate to the
		 * main name, so are at +1 level from the name level. */
		formatGedcomItem(sb,level+1,"NPFX",prefixName);
		formatGedcomItem(sb,level+1,"NSFX",suffixName);
		formatGedcomItem(sb,level,"SEX",sex);
		formatGedcomEvent(sb,level,"BIRT",birth);
		formatGedcomEvent(sb,level,"DEAT",death);
		formatGedcomEvent(sb,level,"BURI",burial);
		formatGedcomReference(sb,level,"FAMC",parents);
		RootsRecord m[] = findReference("marriage");
		if (m!=null) {
			for (int i=0; i<m.length; i++)
				formatGedcomReference(sb,level,"FAMS",m[i]);
		}
		RootsRecord c[] = findReference("children");
		if (c!=null) {
			for (int i=0; i<c.length; i++)
				formatGedcomReference(sb,level,"CHIL",c[i]);
		}
		formatGedcomReference(sb,level,"ADDR",address);
		super.formatGedcomBody(sb,level);
	}

	public String formatNcvType() {
		return ("T:I");
	}
	/** Print out the body of a person record */
	public void formatNcvBody(StringBuffer sb) {
		formatNcvItem(sb,"LN",lastName);
		formatNcvItem(sb,"LN.A",adoptedLastName);
		formatNcvItem(sb,"PN",prefixName);
		formatNcvItem(sb,"FN",firstName);
		formatNcvItem(sb,"MN",middleName);
		formatNcvItem(sb,"NN",nickName);
		formatNcvItem(sb,"SN",suffixName);
		formatNcvItem(sb,"SX",sex);
		formatNcvItem(sb,"B",birth.date);
		formatNcvItem(sb,"BP",birth.place);
		formatNcvItem(sb,"D",death.date);
		formatNcvItem(sb,"DP",death.place);
		formatNcvItem(sb,"BUR",burial.place);
		formatNcvItem(sb,"P",parentsString);
		formatNcvItem(sb,"S",spousesString);
		formatNcvItem(sb,"ADDR",addressString);
		super.formatNcvBody(sb);
	}
	public String formatOneLine(PrintOptions po) {
		StringBuffer sb = new StringBuffer();
		if (po.includeKeys)
			formatKey(sb);
		formatName(sb);
		formatLabeledItems(sb,"b:",birth.date,birth.place," ");
		formatLabeledItems(sb,"d:",death.date,death.place," ");
		return sb.toString();
	}
	public String formatBrief(PrintOptions po) {
		StringBuffer sb = new StringBuffer();
		if (po.includeKeys)
			formatKey(sb);
		formatName(sb);
		sb.append("\n");
		formatLabeledItems(sb,"b: ",birth.date,birth.place,"\n");
		formatLabeledItems(sb,"d: ",death.date,death.place,"\n");
		formatLabeledItem(sb,"buried: ",burial.place,"\n");
		formatNote1(sb,"","\n");
		return sb.toString();
	}
	public String formatName() {
		StringBuffer sb = new StringBuffer();
		formatName(sb);
		return sb.toString();
	}
	public void formatName(StringBuffer sb) {
		if (prefixName!=null)
			sb.append(prefixName).append(" ");
		if (firstName!=null)
			sb.append(firstName).append(" ");
		if (middleName!=null)
			sb.append(middleName).append(" ");
		if (nickName!=null)
			sb.append("(").append(nickName).append(") ");
		if (lastName!=null)
			sb.append(lastName).append(" ");
		if (adoptedLastName!=null)
			sb.append(adoptedLastName).append(" ");
		if (suffixName!=null)
			sb.append(suffixName).append(" ");
	}

	/** Return a string with the family name and an underline. */
	public String formatFamily(PrintOptions po) {
		StringBuffer sb = new StringBuffer();
		String ln = (lastName==null)?"???":lastName;
		sb.append(ln).append('\n');
		String sep = "===============================";
		sb.append(sep.substring(ln.length())).append('\n');
		return sb.toString();
	}

	/** Fill in the layout with information for a ctree
	 * for this record plus other required for the ctree layout.
	 */
	public void layoutCtree(Layout layout, PrintOptions po) {
		/* Start with our own box. */
		layoutBox(layout,po,key);

		/* Add other boxes. */
		layoutBoxReference(layout,po,"marriage");
		layoutBoxReference(layout,po,"parents");
	}

	/** Fill in the layout for a ctree for just this record.
	 */
	protected void layoutBox(Layout layout, PrintOptions po,
						String markKey){
		RootsRecord[] rr = findReference("parentunion");
		if (rr!=null && rr.length>=1) {
			/* found a parent family, use it for the box. */
			RootsUnion ru = (RootsUnion)(rr[0]);
			ru.layoutBox(layout,po,markKey);
		} else {
			/* No parent union, so create a little box for us. */
			RootsBox box = new RootsBox(layout,key);
			String familyText = formatFamily(po);
			box.addSection(familyText,null);
			String thisText = formatBrief(po);
			box.addSection(thisText,key);
			box.done();
		}
	}

	/** Find the specified reference relative to this record.
	 * @param what Relative reference name:
	 * <ul>
	 * <li>parentunion or punion
	 * <li>parents - father and mother
	 * <li>father
	 * <li>mother
	 * <li>address
	 * <li>marriage
	 * <li>union - same as marriage
	 * <li>husband - all husbands
	 * <li>wife - all wives
	 * <li>children or child - all children
	 * <li>marriageN, unionN, husbandN, wifeN - the Nth marriage or spouse,
	 *     where N starts at 0.
	 * <li>childN or childrenN - the Nth child from every union,
	 *     where N starts at 0.
	 * </ul>
	 */
	public RootsRecord[] findReference(String what) {
		if (what==null)
			return null;
		if (what.equals("parentunion") || what.equals("punion")) {
			if (parents==null)
				return null;
			return parents.asArray();
		}
		if (what.equals("parents")) {
			if (parents==null)
				return null;
			RootsRecord f[] = parents.findReference("father");
			RootsRecord m[] = parents.findReference("mother");
			return RootsVector.Combine(m,f);
		}
		if (what.equals("father") || what.equals("mother")) {
			if (parents==null)
				return null;
			return parents.findReference(what);
		}
		if (what.equals("address")) {
			if (address==null)
				return null;
			return address.asArray();
		}
		if (what.startsWith("marriage"))
			return findListReference(spouses,what.substring(8));
		if (what.startsWith("union"))
			return findListReference(spouses,what.substring(5));
		if (what.startsWith("husband")) {
			RootsRecord sa[];
			sa = findListReference(spouses,what.substring(7));
			if (sa==null)
				return null;
			return base.findSelection("husband",sa);
		}
		if (what.startsWith("wife")) {
			RootsRecord sa[];
			sa = findListReference(spouses,what.substring(4));
			if (sa==null)
				return null;
			return base.findSelection("wife",sa);
		}
		if (what.startsWith("children") || what.startsWith("child")) {
			return base.findSelection(what,spouses);
		}
		return super.findReference(what);
	}

	/** Return true if the specified field matches the given pattern.
	 * @param fieldName the name of the field to match
	 * <ul>
	 * <li>"name" to match the full name against the pattern.
	 * <li>"place" to match the name of any locations (birth, death, etc)
	 *     against the pattern.
	 * </ul>
	 * @param pattern the pattern to match against.
	 */
	public boolean fieldMatches(String fieldName, String pattern) {
		if (fieldName.equals("name")) {
			if (fullName==null)
				fullName = formatName();
			return PatternMatch.stringMatches(fullName,pattern);
		}
		if (fieldName.equals("place")) {
			if (PatternMatch.stringMatches(birth.place,pattern))
				return true;
			if (PatternMatch.stringMatches(death.place,pattern))
				return true;
			if (PatternMatch.stringMatches(burial.place,pattern))
				return true;
			return super.fieldMatches(fieldName,pattern);
				/* super may have additional places to match */
		}
		return super.fieldMatches(fieldName,pattern);
	}

	public void println(String s) {
		base.println(s);
	}
}

/* end */
