/* A main window for the roots program.
 *
 * Jim McBeath, June 22, 1997
 */

package jimmc.roots;

import java.awt.*;
import java.awt.event.*;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;

/** A window to control the roots program. */
class RootsWindow extends Frame 
			implements WindowListener, TextPopupListener {
	/** Our command interpreter to be invoked from form actions. */
	CommandInterpreter ci;

	private MenuItem openMenu;
	private MenuItem exitMenu;
	private MenuItem helpAboutMenu;

	private Label titleField;
	private Label fileNameField;
	private Label statusField;
	private TextField commandField;
	private TextField outputFileField;
	private TextField selectField;
	private Button selectCurrentButton;
	private Button selectFatherButton;
	private Button selectMotherButton;
	private Button selectChildrenButton;
	private Button selectMarriageButton;
	private Button selectHusbandButton;
	private Button selectWifeButton;
	private Button selectParentsButton;
	private Button selectParentUnionButton;
	private TextArea outputTextField;
	private PopupMenu outputTextPopup;
	private Button printButton;
	private Choice formatChoice;
	private Choice recordFormatChoice;
	private Checkbox keyCheckbox;
	private Checkbox historyCheckbox;
	private Checkbox reportCheckbox;

	private CardLayout tabPanelsLayout;
	private Panel tabPanels;
	private Panel reportPanel;
	private Panel historyPanel;

	/** Create a roots window with a title. */
	public RootsWindow(String title) {
		super(title);
		init();
		addWindowListener(this);
	}

	/* Finish the constructor initialization. */
	private void init() {
		RootsBase b = new RootsBase();
			/* Initialize an empty database.  Let the user
			 * fill it in with his data file using the load
			 * command in the CommandInterpreter. */
		ci = new CommandInterpreter(b);
		setupMenu();
		initForm();
	}
	private void setupMenu() {
		MenuBar mb = new MenuBar();

		Menu m = new Menu("File");
		m.add(openMenu=new MenuItem("Open"));
		m.add(exitMenu=new MenuItem("Exit"));
		mb.add(m);

		m = new Menu("Help");
		m.add(helpAboutMenu=new MenuItem("About Roots..."));
		mb.add(m);
		mb.setHelpMenu(m);

		setMenuBar(mb);
	}
	private void initForm() {
		GridBagger gb=new GridBagger(this);

		/* Set up the top half of the window. */
		gb.add(createTopPanel());
		gb.nextRow();

		/* Set up a checkbox group to select what to view in
		 * the bottom half of the form. */
		Panel tabs = new Panel();
		tabs.setLayout(new FlowLayout());
		CheckboxGroup cbg = new CheckboxGroup();
		tabs.add(historyCheckbox=new Checkbox("History",cbg,true));
		tabs.add(reportCheckbox=new Checkbox("Report",cbg,false));
		gb.add(tabs);
		gb.nextRow();

		/* Set up the panels under the checkbox. */
		tabPanels = new Panel();
		tabPanels.setLayout(tabPanelsLayout=new CardLayout());
		tabPanels.add("history",historyPanel=createHistoryPanel());
		tabPanels.add("report",reportPanel=createReportPanel());
		gb.add(tabPanels);

		/* End with a status line. */
		gb.nextRow();
		gb.gbc.fill=GridBagConstraints.HORIZONTAL;
		gb.addLast(statusField=new Label(" "));
		pack();
	}
	private Panel createTopPanel() {
		Panel p = new Panel();
		GridBagger gb = new GridBagger(p);
		gb.add(titleField=new Label("Roots"));
		gb.gbc.fill=GridBagConstraints.HORIZONTAL;
		gb.addLast(fileNameField=new Label(" "));
		gb.gbc.fill=GridBagConstraints.NONE;
		gb.nextRow();
		gb.add(new Label("Command:"));
		gb.addLast(commandField=new TextField(40));
		gb.nextRow();
		gb.add(new Label("Select:"));
		gb.addLast(selectField=new TextField(40));
		gb.nextRow(); gb.gbc.gridx++;
		gb.add(selectCurrentButton=new Button("Current"));
		gb.add(selectFatherButton=new Button("Father"));
		gb.add(selectMotherButton=new Button("Mother"));
		gb.add(selectChildrenButton=new Button("Children"));
		gb.nextRow(); gb.gbc.gridx++;
		gb.add(selectMarriageButton=new Button("Marriage"));
		gb.add(selectHusbandButton=new Button("Husband"));
		gb.add(selectWifeButton=new Button("Wife"));
		gb.add(selectParentsButton=new Button("Parents"));
		gb.nextRow(); gb.gbc.gridx++;
		gb.add(selectParentUnionButton=new Button("ParentUnion"));
		gb.nextRow();
		return p;
	}
	private Panel createReportPanel() {
		Panel p = new Panel();
		GridBagger gb = new GridBagger(p);

		gb.add(printButton=new Button("Print"));
		gb.add(new Label("Format:"));
		formatChoice = new Choice();
		String[] formats = ci.getPrintFormats();
		for (int i=0; i<formats.length; i++)
			formatChoice.addItem(formats[i]);
		gb.add(formatChoice);
		gb.nextRow(); gb.gbc.gridx++;
		gb.add(new Label("Record format:"));
		recordFormatChoice = new Choice();
		recordFormatChoice.addItem("");
		String[] recordFormats = ci.getRecordFormats();
		for (int i=0; i<recordFormats.length; i++)
			recordFormatChoice.addItem(recordFormats[i]);
		gb.add(recordFormatChoice);
		gb.nextRow(); gb.gbc.gridx++;
		gb.add(keyCheckbox=new Checkbox("Include keys"));
		gb.nextRow(); gb.gbc.gridx++;
		gb.add(new Label("Output:"));
		gb.add(outputFileField=new TextField(40));
		gb.nextRow();
		GridBagConstraints gbc2 = (GridBagConstraints)gb.gbc.clone();
		gbc2.gridwidth=GridBagConstraints.REMAINDER;
		gbc2.fill=GridBagConstraints.BOTH;
		gbc2.weightx=1;
		gbc2.weighty=1;
		outputTextPopup = new TextPopup(this);
		p.add(outputTextPopup);
		gb.add(gbc2,outputTextField=new TextAreaWithPopup(
					outputTextPopup,10,40) {
			public Dimension getPreferredSize() {
				setFixedFont();
				return super.getPreferredSize();
			}
			protected void showPopupMenu(PopupMenu p, Component c,
						int x, int y) {
				TextPopup tp = (TextPopup)p;
				tp.setRecord(this,x,y);
				super.showPopupMenu(p,c,x,y);
			}
		});
		return p;
	}
	private Panel createHistoryPanel() {
		Panel p = new Panel();
		p.add(new Label("no history yet"));
		return p;
	}
	public Dimension getPreferredSize() {
		setBoldFont(titleField);
		return super.getPreferredSize();
	}
	/* Set the font of the specified Component to a bold version of
	 * whatever is currently set.
	 * This method can't be called until the component is realized;
	 * calling it from preferredSize() works well.
	 */
	private void setBoldFont(Component c) {
		Font f = c.getFont();
		int newStyle = f.getStyle()|Font.BOLD;
		Font newFont = new Font(f.getFamily(),newStyle,f.getSize());
		c.setFont(newFont);
	}
	public boolean action(Event e, Object o) {
		if (e.target instanceof MenuItem) {
			MenuItem m = (MenuItem)e.target;
			if (m==exitMenu) {
				System.exit(0);
				/* NOTREACHED */
			}
			if (m==openMenu) {
				processOpen();
				return true;
			}
			if (m==helpAboutMenu) {
				processHelpAbout();
				return true;
			}
		}
		if (e.target instanceof TextField) {
			TextField f = (TextField)e.target;
			if (f==commandField) {
				processCommand();
				return true;
			}
			if (f==selectField) {
				processSelect();
				return true;
			}
		}
		if (e.target instanceof Button) {
			Button b = (Button)e.target;
			if (b==printButton) {
				processPrint();
				return true;
			}
			if (b==selectCurrentButton) {
				processSelect("current");
				return true;
			}
			if (b==selectFatherButton) {
				processSelect("father");
				return true;
			}
			if (b==selectMotherButton) {
				processSelect("mother");
				return true;
			}
			if (b==selectChildrenButton) {
				processSelect("children");
				return true;
			}
			if (b==selectMarriageButton) {
				processSelect("marriage");
				return true;
			}
			if (b==selectHusbandButton) {
				processSelect("husband");
				return true;
			}
			if (b==selectWifeButton) {
				processSelect("wife");
				return true;
			}
			if (b==selectParentsButton) {
				processSelect("parents");
				return true;
			}
			if (b==selectParentUnionButton) {
				processSelect("parentunion");
				return true;
			}
		}
		if (e.target instanceof Checkbox) {
			Checkbox b = (Checkbox)e.target;
			if (b.getState()==false)
				return true;	/* ignore turn-off */
			if (b==historyCheckbox) {
				processShowHistory();
				return true;
			}
			if (b==reportCheckbox) {
				processShowReport();
				return true;
			}
		}
		return false;
	}
	private void processHelpAbout() {
		AboutWindow.showAboutWindow(this);
	}
	private void processOpen() {
		FileDialog d = new FileDialog(this,"Load File");
		d.show();		/* get filename from user */
		String dir = d.getDirectory();
		String fn = d.getFile();
		if (fn==null || fn.equals("")) {
			setStatus("Cancelled");
			return;
		}
		loadFile(dir+fn);
	}
	private void processCommand() {
		String cmd = commandField.getText().trim();
		doCommand(cmd);
	}
	private void doCommand(String cmd) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		PrintWriter ps = new PrintWriter(baos);
		PrintWriter oldp = ci.base.defaultPrintOptions.out;
		PrintWriter olds = ci.base.defaultShowOptions.out;
		ci.base.defaultPrintOptions.out = ps;
		ci.base.defaultShowOptions.out = ps;
		commandField.setText(cmd);
		ci.doCommand(cmd);
		outputTextField.setText(baos.toString());
		ci.base.defaultPrintOptions.out = oldp;
		ci.base.defaultShowOptions.out = olds;
	}
	private void loadFile(String inputFileName) {
		setStatus("loading "+inputFileName);
		doCommand("load "+inputFileName);
		setFileName(inputFileName);
		clearStatus();
	}
	private void processSelect() {
		String select = selectField.getText().trim();
		processSelect(select);
	}
	public void processSelect(String select) {
		doCommand("select "+select);
		selectField.setText(ci.base.getSelectedKeysString());
	}
	private void processPrint() {
		StringBuffer sb = new StringBuffer();
		sb.append("print ");
		/* If output file specified, print to that file */
		String outputFileName = outputFileField.getText().trim();
		if (!outputFileName.equals(""))
			sb.append("-o ").append(outputFileName).append(" ");
		if (keyCheckbox.getState())
			sb.append("-k ");
		/* Add the format */
		String format = formatChoice.getSelectedItem();
		sb.append("-f ").append(format).append(" ");
		/* Add the record format */
		String recordFormat = recordFormatChoice.getSelectedItem();
		if (!recordFormat.equals(""))
			sb.append("-r ").append(recordFormat).append(" ");
		/* Add the selection */
		sb.append(selectField.getText().trim());
		String cmd = sb.toString();
		commandField.setText(cmd);
		doCommand(cmd);
	}
	private void processShowHistory() {
		tabPanelsLayout.show(tabPanels,"history");
	}
	private void processShowReport() {
		tabPanelsLayout.show(tabPanels,"report");
	}
	private void setFileName(String fn) {
		fileNameField.setText(fn);
	}
	private void setStatus(String s) {
		statusField.setText(s);
	}
	private void clearStatus() {
		setStatus("");
	}

	/* The WindowListern interface */
	public void windowOpened(WindowEvent e){}
	public void windowClosing(WindowEvent e){
		dispose();	/* quit */
		System.exit(0);
	}
	public void windowClosed(WindowEvent e){
		System.out.println("windowClosed");
	}
	public void windowIconified(WindowEvent e){}
	public void windowDeiconified(WindowEvent e){}
	public void windowActivated(WindowEvent e){}
	public void windowDeactivated(WindowEvent e){}

	/** Main routine to bring up the RootsWindow screen. */
	public static void main(String[] argv) {
		RootsWindow w = new RootsWindow("Roots");
		w.show();
	}
}

/* end */
