/* SingleModel.java
 *
 * Jim McBeath, June 7, 1998
 */

package jimmc.roots;

import jimmc.treepar.Box;
import jimmc.treepar.Connector;
import jimmc.treepar.Layout;

import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;

/** A model for a single record from the database.
 */
public class SingleModel {
	/** Our database model. */
	protected DatabaseModel db;
	/** Our key. */
	protected String key;
	/** Our base database. */
	protected RootsBase base;
	/** Our record. */
	protected RootsRecord record;

	/** Create a new model. */
	public SingleModel(DatabaseModel db, String key) {
		this.db = db;
		this.key = key;
		base = db.getBase();
		record = base.findRecord(key);
		/* TBD - check for null record? */
	}

	/** Get a selection relative to this selection.
	 * @param what the selction, such as father, mother, or children.
	 */
	public String[] getSelection(String what) {
		db.select(what+" of "+key);
		return db.getSelectedKeys();
	}

	/** Get the key for this model. */
	public String getKey() {
		return key;
	}

	/** Get the type for this model. */
	public String getType() {
		if (record==null)
			return "(none)";
		return record.formatNcvType();	/* TBD - do better */
	}

	/** Get the summary line to display for this model. */
	public String getSummary() {
		if (record==null)
			return "no such record";
		return record.formatOneLine(new PrintOptions());
	}

	/** Get the command we use to generate the display info. */
	public String getDisplayCommand(String format, boolean includeKeys) {
		if (format.equals("Tctree")) {
			/* ctree is a graphics format; for debug, return the
			 * text dump for it. */
			return getTreeparText(format,includeKeys);
		}

		StringBuffer sb = new StringBuffer();
		sb.append("print ");
		if (includeKeys)
			sb.append("-k ");
		if (format.equals("gedcom"))
			sb.append("-f list -r gedcom ");
		else if (format.equals("ncv"))
			sb.append("-f list ");	//default record format is ncv
		else
			sb.append("-f ").append(format).append(" ");
		if (format.equals("atree"))
			sb.append("plus recurse parents of ");
		else if (format.equals("dtree"))
			sb.append("plus recurse children of ");
		sb.append(key);		/* our record */
		return sb.toString();
	}

	/** Get the long text string based on format. */
	public String getOutputText(String format, boolean includeKeys) {
		String cmd = getDisplayCommand(format,includeKeys);
		return db.doCommand(cmd);
	}

	/** Get the text dump of a treepar layout. */
	protected String getTreeparText(String format, boolean includeKeys) {
		Layout layout = getOutputLayout(format, includeKeys);

		/* Create a string output stream to write to. */
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		PrintWriter ps = new PrintWriter(baos);
		layout.dump(ps);	/* dump to the string */
		return baos.toString();	/* return the dump */
	}

	/** Get the layout and route it. */
	protected Layout getOutputLayout(String format, boolean includeKeys) {
		Layout layout = new Layout(true);

		PrintOptions po = new PrintOptions();
		po.includeKeys = includeKeys;

		if (format.equals("Gctree")||format.equals("Tctree"))
			record.layoutCtree(layout,po);
		else
			throw new RuntimeException(
				"Unknown layout format "+format);

		/* TBD - add layout.setRootBox(firstBox);
		 * add method to Layout.java */
		//layout.placeAndRoute(null,Layout.STEP_ROW_NUMBERS);
		layout.placeAndRoute();
		return layout;
	}

	/** Debug - Fill in the layout with boxes. */
	protected void fillLayout(Layout layout) {
		Box box;
		Connector conn;

		box = new Box(layout,"b1");
		box.setSize(40,60);
		box.setText("This is b1");
		conn = new Connector(box,"net1");
		conn.setSide(Layout.EAST);
		conn.setPosition(40,30);

		box = new Box(layout,"b2");
		box.setSize(45,50);
		box.setText("Box b2");
		conn = new Connector(box,"net1");
		conn.setSide(Layout.WEST);
		conn.setPosition(0,20);
	}
}

/* end */
