/* SingleView.java
 *
 * Jim McBeath, June 7, 1998
 */

package jimmc.roots;

import jimmc.treepar.Layout;
import jimmc.treepar.LayoutPanel;

import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.GridBagConstraints;

/** SingleView provides a view into a single record.
 */
public class SingleView extends RootsFrame {
	/** Our database. */
	protected DatabaseModel databaseModel;

	/** Our model. */
	protected SingleModel model;

	/** Where we display the key value for our record. */
	protected JLabel keyField;

	/** Where we display the type of our record. */
	protected JLabel typeField;

	/** Where we display the one-line summary of our record. */
	protected JLabel summaryField;

	/** Contains either the text panel or the layout panel. */
	protected TextOrLayoutPanel outputPanel;

	/** The output type currently selected. */
	protected JComboBox outputChoiceField;

	/** The include-keys toggle. */
	protected JCheckBox includeKeysField;

	/** Create a new view for a record. */
	public SingleView(DatabaseModel db, String key) {
		super(key);
		this.databaseModel = db;
		setJMenuBar(createMenuBar());
		initForm();
		addWindowListener();
		setModel(databaseModel.getSingleModel(key));
		pack();
	}

	/** Set the model we are viewing. */
	public void setModel(SingleModel m) {
		model = m;
		keyField.setText(m.getKey());
		typeField.setText(m.getType());
		summaryField.setText(m.getSummary());
		setOutputDisplay();
		validate();
	}

	/** Set the output field. */
	protected void setOutputDisplay() {
		if (includeKeysField==null)
			return;	/* this happens during init */
		boolean includeKeys = includeKeysField.isSelected();
		String outputFormat =
			(String)outputChoiceField.getSelectedItem();
		if (outputFormat.startsWith("G")) {
			Layout layout = model.getOutputLayout(
					outputFormat,includeKeys);
			outputPanel.setLayout(layout);
		} else {
			String text = model.getOutputText(
					outputFormat,includeKeys);
			outputPanel.setText(text);
		}
	}

	/** Create our menu bar. */
	protected JMenuBar createMenuBar() {
		JMenuBar mb = new JMenuBar();
		mb.add(createFileMenu());
		//JMenu helpMenu = createHelpMenu();
		//mb.add(helpMenu);
		//mb.setHelpMenu(helpMenu);
		return mb;
		/* TBD - add a select menu with things like ancestors to
		 * N generations, descendants to N generations, whatever
		 * else seems reasonable.
		 */
		/* TBD - add a menu command to save the large text area
		 * to a file. */
	}

	/** Create our File menu. */
	protected JMenu createFileMenu() {
		JMenu m = new JMenu("File");
		MenuAction mi;
		mi = new MenuAction("Save Output As...") {
			public void action() {
				processSaveOutputAs();
			}
		};
		m.add(mi);
		mi = new MenuAction("Show Display Command") {
			public void action() {
				processShowDisplayCommand();
			}
		};
		m.add(mi);
		mi = new MenuAction("Close") {
			public void action() {
				processClose();
			}
		};
		m.add(mi);
		return m;
	}

	/** Create the body of our form. */
	protected void initForm() {
		GridBagger gb = new GridBagger(getContentPane());
		gb.gbc.fill = GridBagConstraints.HORIZONTAL;
		gb.gbc.anchor = GridBagConstraints.WEST;
		gb.add(createInfoArea());
		gb.nextRow();
		gb.add(createSelectionArea());
		gb.nextRow();
		gb.gbc.weightx = 1;
		gb.gbc.weighty = 1;
		gb.gbc.fill = GridBagConstraints.BOTH;
		gb.add(createOutputArea());
	}
	
	/** Create the info box at the top of the form. */
	protected Component createInfoArea() {
		JPanel p = new JPanel();
		p.setBorder(new TitledBorder(new EtchedBorder(),"Record"));
		GridBagger gb = new GridBagger(p);
		gb.gbc.fill = GridBagConstraints.HORIZONTAL;
		gb.gbc.anchor = GridBagConstraints.WEST;
		gb.add(new JLabel("Key:"));
		gb.add(keyField=new JLabel());
		gb.nextRow();
		gb.add(new JLabel("Type:"));
		gb.add(typeField=new JLabel());
		gb.nextRow();
		gb.add(new JLabel("Summary:"));
		gb.gbc.weightx = 1;
		gb.add(summaryField=new JLabel());
		return p;
	}

	/** Create the selection buttons. */
	protected Component createSelectionArea() {
		JPanel p = new JPanel();
		p.setBorder(new TitledBorder(new EtchedBorder(),"Select"));
		GridBagger gb = new GridBagger(p);
		/* TBD - put in a general purpose select text box to allow
		 * easy selection of things like "plus recurse4 father"
		 */
		gb.gbc.fill = GridBagConstraints.HORIZONTAL;
		gb.add(new SelectButton("Father"));
		gb.add(new SelectButton("Mother"));
		gb.add(new SelectButton("Parents"));
		gb.add(new SelectButton("+Ancestors","plus recurse parents"));
		gb.gbc.weightx = 1;
		gb.add(new JLabel(""));	/* filler column */
		gb.gbc.weightx = 0;
		gb.nextRow();
		//gb.gbc.gridx++;
		gb.add(new SelectButton("Husband"));
		gb.add(new SelectButton("Wife"));
		gb.add(new SelectButton("Children"));
		gb.add(new SelectButton(
				"+Descendents","plus recurse children"));
		gb.nextRow();
		//gb.gbc.gridx++;
		gb.add(new SelectButton("Marriage"));
		gb.add(new SelectButton("ParentUnion"));
		return p;
	}

	/** Create the output area. */
	protected Component createOutputArea() {
		JPanel p = new JPanel();
		p.setBorder(new TitledBorder(new EtchedBorder(),"Output"));
		GridBagger gb = new GridBagger(p);
		gb.add(new JLabel("Display:"));
		outputChoiceField = new ComboBoxAction() {
			public void action() {
				setOutputDisplay();
			}
		};
		outputChoiceField.addItem("atree");
		outputChoiceField.addItem("dtree");
		outputChoiceField.addItem("fampage");
		outputChoiceField.addItem("gedcom");
		outputChoiceField.addItem("ncv");
		outputChoiceField.addItem("Tctree");
		outputChoiceField.addItem("Gctree");
		outputChoiceField.setSelectedItem("ncv");	/* default */
		gb.add(outputChoiceField);
		includeKeysField = new CheckBoxAction("Include Keys") {
			public void action() {
				setOutputDisplay();
			}
		};
		gb.add(includeKeysField);
		gb.nextRow();
		gb.gbc.gridwidth = 4;
		gb.gbc.weightx = 1;
		gb.gbc.weighty = 1;
		gb.gbc.fill = GridBagConstraints.BOTH;
		outputPanel = new TextOrLayoutPanel() {
			public void processTextSelect(String key) {
				SingleView.this.processSelect(key);
			}
			public Dimension getPreferredSize() {
				return new Dimension(500,300);
			}
		};
		gb.add(outputPanel);

		return p;
	}

	/** Change the selection. */
	public void processSelect(String sel) {
		String[] newSel = model.getSelection(sel);
		if (newSel==null || newSel.length==0) {
			errorDialog("No match");
				/* nothing selected, don't switch */
		} else if (newSel.length==1) {
			setModel(databaseModel.getSingleModel(newSel[0]));
		} else {
			/* multiple items, put up a choice list
			 * and let him pick one or cancel,
			 * or open up a ListView with the items.
			 */
			String oneSel = processListSelection(newSel);
			/* If he picked one of them, switch to that record;
			 * else ignore it. */
			if (oneSel!=null)
				setModel(databaseModel.getSingleModel(oneSel));
		}
	}

	/** Put up a choicelist of items, let the user select one.
	 * @return the selected key, or null if no key was selected.
	 *         This happens if the user Cancels, or selects the option
	 *         to open up a ListView.
	 */
	protected String processListSelection(String[] keys) {
		String prompt = "Select which record to switch to.\n"+
			"Number of records: "+keys.length;
		String[] keySummaries = databaseModel.getKeySummaries(keys);
		JList list = new JList(keySummaries);
		JScrollPane pane = new JScrollPane(list);
		Object[] msgs = { prompt, pane };
		Object[] options = { "OK", "Cancel", "ListView" };
		int n = JOptionPane.showOptionDialog(this,msgs,"Select",
			JOptionPane.DEFAULT_OPTION,
			JOptionPane.QUESTION_MESSAGE,null,options,options[0]);
		if (n==0)
			return keys[list.getSelectedIndex()];
		if (n==2)
			(new ListView(databaseModel,keys)).show();
		return null;
	}

	/** Process the File->ShowDisplayCommand command. */
	protected void processShowDisplayCommand() {
		boolean includeKeys = includeKeysField.isSelected();
		String outputFormat =
			(String)outputChoiceField.getSelectedItem();
		String text = model.getDisplayCommand(outputFormat,includeKeys);
		infoDialog(text);
	}

	/** Process the File->SaveOutputAs command. */
	protected void processSaveOutputAs() {
		/* TBD - check if using layout instead of text. */
		String text = outputPanel.getText();
		if (text.length()==0) {
			errorDialog("No text to save.");
			return;
		}
		String filename = fileDialog("Save to file:");
		if (filename==null)
			return;
		saveTextToFile(text,filename);
	}

	/** Close the window. */
	protected void processClose() {
		/* TBD - notify the model. */
		super.processClose();
	}

	/* A button to change the selection */
	class SelectButton extends ButtonAction {
		String sel;
		public SelectButton(String label) {
			this(label,label.toLowerCase());
		}
		public SelectButton(String label, String sel) {
			super(label);
			this.sel = sel;
		}
		public void action () {
			SingleView.this.processSelect(sel);
		}
	}
}

/* end */
