/* Connector.java
 *
 * Jim McBeath, July 8, 1998
 */

package jimmc.treepar;

import jimmc.roots.Sort;
import jimmc.roots.SortComparator;

import java.awt.Point;
import java.io.PrintWriter;
import java.util.Vector;

/** Connector represents on connector on the edge of a Box.
 * @see Box
 */
public class Connector {
	/* These variables are used as input to the layout process. */
	/** Our Box. */
	protected Box box;
	/** The name of this connector. */
	protected String name;
	/** The location of this connector relative to the box origin. */
	protected Point position;
	/** The edge of the box on which this connector lies. */
	protected int side;
	/** The name of the net this connector attaches to. */
	protected String netName;

	/* These variables are calculated during the layout process. */
	/** The net to which this connector is attached. */
	protected Net net;

	/** Create a new Connector on a Box. */
	public Connector(Box box, String name) {
		this.box = box;
		this.name = name;
		netName = name;		/* default net name is same as name */
		box.addConnector(this);
		position = new Point(0,0);
	}

	/** Set the netName of this connector. */
	public void setNetName(String name) {
		netName = name;
	}

	/** Set the location of this connector relative to the box origin. */
	public void setPosition(int x, int y) {
		position.x = x;
		position.y = y;
	}

	/** Adjust the relative location of this connector. */
	public void movePosition(int dx, int dy) {
		position.x += dx;
		position.y += dy;
	}

	/** Set the side this connector is on. */
	public void setSide(int side) {
		this.side = side;
	}

	/** Get the side this connector is on. */
	public int getSide() {
		return side;
	}

	/** Propagate the row number from a
	 * <a href="jimmc.treepar.Box.html">Box</a>
	 * through this connector to our
	 * <a href="jimmc.treepar.Net.html">Net</a>.
	 * @param rowNum the row number of the box.
	 */
	protected void setRowFromBox(int rowNum) {
		if (net==null)
			return;		/* no attached net */
		switch (side) {
		case Layout.NORTH:
		case Layout.EAST:
			net.setRowNumber(rowNum+1);
			break;
		case Layout.SOUTH:
		case Layout.WEST:
			net.setRowNumber(rowNum-1);
			break;
		}
	}

	/** Propagate the row number from a
	 * <a href="jimmc.treepar.Net.html">Net</a>
	 * through this connector to our
	 * <a href="jimmc.treepar.Box.html">Box</a>.
	 * @param rowNum the row number of the net.
	 */
	protected void setRowFromNet(int rowNum) {
		if (box==null)
			return;		/* no attached box */
		switch (side) {
		case Layout.NORTH:
		case Layout.EAST:
			box.setRowNumber(rowNum-1);
			break;
		case Layout.SOUTH:
		case Layout.WEST:
			box.setRowNumber(rowNum+1);
			break;
		}
	}

	/** Link our net. */
	protected void linkNet() {
		net = box.layout.findOrCreateNet(netName);
		net.addConnector(this);
	}

	/** Get the R coordinate of the connector relative to its box. */
	protected int getR() {
		return (box.layout.verticalRows)?position.y:position.x;
	}

	/** Get the absolute R coordinate. */
	protected int getAbsoluteR() {
		return (box.layout.verticalRows)?
			box.origin.y+position.y:
			box.origin.x+position.x;
	}

	/** Get the absolute X coordinate. */
	protected int getAbsoluteX() {
		return box.origin.x+position.x;
	}

	/** Get the absolute Y coordinate. */
	protected int getAbsoluteY() {
		return box.origin.y+position.y;
	}

	/** Get the absolute X and Y coordinates. */
	protected Point getAbsoluteLocation() {
		return new Point(box.origin.x+position.x,
				box.origin.y+position.y);
	}

	/** Get all of the connectors on the specified side of their box,
	 * sort them by position, and return an array.
	 * @param connectors the list of connectors through which to search.
	 * @param side the side for which connectors are desired.
	 * @return array of connectors sorted by position, or null if not
	 *         connectors were on that side of the box.
	 */
	protected static Connector[] getSideConnectors(Connector[] connectors,
						int side) {
		Vector cv = new Vector();	/* collect conns on this side */
		for (int c=0; c<connectors.length; c++) {
			if (connectors[c].getSide()==side)
				cv.addElement(connectors[c]);
		}
		if (cv.size()==0)
			return null;		/* no connectors on this side */
		Connector[] cc = new Connector[cv.size()];
		cv.copyInto(cc);
		/* Sort the list by position */
		Sort.sortArray(cc,new SortConnectors());
		return cc;
	}

	/** Dump a connector for debugging. */
	public void dump(PrintWriter f, String prefix, boolean details) {
		f.println(prefix+"Connector: "+box.name+"."+name+ "("+
				net.name+")");
		if (!details)
			return;
		Point apos = new Point(box.origin.x+position.x,
					box.origin.y+position.y);
		f.println(prefix+"  side="+side+", pos="+position+
					", apos="+apos);
	}
	
	/** Dump an array of connectors. */
	public static void dump(PrintWriter f, Connector[] connectors,
					String prefix, boolean details) {
		f.print(prefix);
		f.print("Connectors: ");
		if (connectors==null) {
			f.println("null");
			return;
		}
		f.println(connectors.length);
		for (int c=0; c<connectors.length; c++) {
			connectors[c].dump(f,prefix+" ["+c+"] ",details);
		}
	}
}

/* A class to sort connectors. */
class SortConnectors implements SortComparator {
	/** Compare two connectors, sort on R coordinate. */
	public int compare(Object c1, Object c2) {
		return ((Connector)c1).getR() - ((Connector)c2).getR();
	}
}

/* end */
