/* Net.java
 *
 * Jim McBeath, July 8, 1998
 */

package jimmc.treepar;

import jimmc.roots.Sort;
import jimmc.roots.SortComparator;

import java.awt.Point;
import java.io.PrintWriter;
import java.util.Vector;

/** Net connects together the Connector objects from multiple Box objects.
 */
public class Net {
	/** Our layout. */
	protected Layout layout;

	/** Our name. */
	protected String name;

	/** The connectors attached to this net, as we build it. */
	protected Vector connectorsVector;

	/** The connectors attached to this net. */
	protected Connector[] connectors;

	/** Our row number. */
	protected int rowNumber;

	/** True when rowNumber is set. */
	protected boolean rowNumberSet;

	/** The row containing this net. */
	protected Row row;

	/** The ordinal number of our position within our row. */
	protected int rowOrder;

	/** True when the row order has been set for this net. */
	protected boolean orderSet;

	/** The minimum R coordinate of the routing span. */
	protected int minR;

	/** The maximum R coordinate of the routing span. */
	protected int maxR;

	/** The routing angle for this net. */
	protected int angle;

		public static int LEFT = 4;
		public static int RIGHT = 5;

	/** The routing track to use for this net. */
	protected Track track;

	/** The points comprising our routing path.
	 * Each paths[n] is an array of points for one path.
	 */
	protected Point[][] paths;

	/** Create a new net. */
	public Net(Layout layout, String name) {
		this.layout = layout;
		this.name = name;
		clearConnectors();
	}

	/** Clear the connectors for the net. */
	protected void clearConnectors() {
		connectorsVector = new Vector();
		connectors = null;
	}

	/** Add a connector to this net. */
	protected void addConnector(Connector conn) {
		connectorsVector.addElement(conn);
		connectors = null;
	}

	/** Note all the connectors have been added. */
	protected void connectorsDone() {
		connectors = new Connector[connectorsVector.size()];
		connectorsVector.copyInto(connectors);
	}

	/** Clear the row number. */
	protected void clearRowNumber() {
		rowNumberSet = false;
		rowNumber = 0;
		row = null;
	}

	/** Set the row number on all boxes connected to this net. */
	protected void setRowNumber(int rowNum) {
		if (rowNumberSet)
			return;
		rowNumber = rowNum;
		rowNumberSet = true;
		for (int c=0; c<connectors.length; c++)
			connectors[c].setRowFromNet(rowNum);
	}

	/** Set the row for this net. */
	protected void setRow(Row row) {
		this.row = row;
	}

	/** Split the net into multiple nets if it spans rows. */
	protected void split() {
		/* TBD - RSplitNet(net) */
	}

	/** Clear the row ordering for this net. */
	protected void clearRowOrder() {
		orderSet = false;
	}

	/** Set the row ordering for this net. */
	protected void setRowOrder(boolean negdir) {
		if (orderSet)
			return;		/* already set */
		orderSet = true;
		if (negdir)
			rowOrder = --row.rowOrderMin;
		else
			rowOrder = row.rowOrderMax++;

		/* set the ordering on the connectors */
		setConnectorsRowOrder(layout.SorW,negdir);
		setConnectorsRowOrder(layout.NorE,negdir);
	}

	/** Return true if the row order has already been set on this net.
	 */
	protected boolean isRowOrderSet() {
		return orderSet;
	}

	/** Set the ordering on the connectors on the specified side
	 * of their boxes.
	 * @param side the side of the box into which connectors
	 *        we propagate.
	 * @param negdir true means new items are added to the lesser
	 *        end of the row.
	 */
	protected void setConnectorsRowOrder(int side, boolean negdir) {
		Connector[] cc = Connector.getSideConnectors(connectors,side);
		if (cc==null)
			return;		/* no connectors on that side. */

		/* See if the box on any of these connectors has been set.
		 * If so, then we don't have a simple tree-structured
		 * network.  We will do our continued order around
		 * that connector in an effort to minimize crossed lines.
		 */
		int c;
		for (c=0; c<cc.length; c++)
			if (cc[c].box.isRowOrderSet())
				break;
		if (c>=cc.length) {
			/* no nets have already been set, just start at
			 * one end and work through them.
			 */
			c = negdir?cc.length:0;
		}

		/* Traverse the boxes on our connectors that are higher than
		 * the previously found net.
		 */
		for (int j=c; j<cc.length; j++)
			cc[j].box.setRowOrder(false);
		/* Traverse the boxes that are lower. */
		for (int j=c-1; j>=0; j--)
			cc[j].box.setRowOrder(true);
	}

	/** Add in the force data from the specified connector to all
	 * other connectors on the net.
	 * The force is the sum of the difference in the R coordinates between
	 * the specified connector and the other connectors on the net.
	 * @param conn the origin connector.
	 * @param forcesums [0] is the force sum, [1] is the count.
	 *        We add to these two numbers.
	 */
	protected void addNetForce(Connector conn, int[] forcesums) {
		int r = conn.getAbsoluteR();
		for (int c=0; c<connectors.length; c++) {
			Connector cc = connectors[c];
			if (cc==conn)
				continue;	/* not against itself */
			forcesums[1]++;		/* count of other connectors */
			int f = cc.getAbsoluteR() - r;
					/* distance along the row */
			forcesums[0] += f;
		}
	}

	/** Calculate the span of the net.
	 * The span is the minimum and maximum values of the R coordinate
	 * of all connectors on the net.
	 */
	protected void calculateSpan() {
		for (int c=0; c<connectors.length; c++) {
			Connector conn = connectors[c];
			int connR = conn.getAbsoluteR();
			if (c==0 || connR > maxR)
				maxR = connR;
			if (c==0 || connR < minR)
				minR = connR;
		}
	}

	/** Calculate the routing angle of the net. */
	protected void calculateRoutingAngle() {
		if (connectors.length<=1)
			return;		/* no routing for this net. */
		int nconn = 0;	/* connector count for each side. */
		int sconn = 0;
		int econn = 0;
		int wconn = 0;
		int nsum = 0;
		int ssum = 0;
		int esum = 0;
		int wsum = 0;
		track = null;
		for (int c=0; c<connectors.length; c++) {
			Connector conn = connectors[c];
			switch (conn.getSide()) {
			case Layout.NORTH:
				nconn++;
				nsum += conn.getAbsoluteX();
				break;
			case Layout.SOUTH:
				sconn++;
				ssum += conn.getAbsoluteX();
				break;
			case Layout.EAST:
				econn++;
				esum += conn.getAbsoluteY();
				break;
			case Layout.WEST:
				wconn++;
				wsum += conn.getAbsoluteY();
				break;
			default:
				break;
			}
		}
		if      (nconn>sconn) angle = Layout.SOUTH;
		else if (sconn>nconn) angle = Layout.NORTH;
		else if (econn>wconn) angle = Layout.WEST;
		else if (wconn>econn) angle = Layout.EAST;
		else if (esum>wsum)   angle = Layout.LEFT;
		else if (wsum>esum)   angle = Layout.RIGHT;
		else if (nsum>ssum)   angle = Layout.LEFT;
		else if (ssum>nsum)   angle = Layout.RIGHT;
		else angle = -1;
	}

	/** Return true if we need to do a track assignment
	 * for this net.
	 */
	protected boolean needsTrackAssignment() {
		if (connectors.length<=1)
			return false;
		switch (angle) {
		case Layout.WEST:
		case Layout.SOUTH:
		case Layout.RIGHT:
			return false;
		}
		return true;
	}

	/** Set the track to use for routing this net. */
	protected void setTrack(Track track) {
		this.track = track;
	}

	/** Route the net. */
	protected void route() {
		if (connectors.length<=1)
			return;		/* no routing needed for this net. */
		int cpos = track.position;
		Point[] path;
		if (connectors.length==2) {
			/* common special case */
			Connector c0 = connectors[0];
			Connector c1 = connectors[1];
			if (minR==maxR) {
				/* a straight line from one conn to the other */
				path = new Point[2];
				path[0] = c0.getAbsoluteLocation();
				path[1] = c1.getAbsoluteLocation();
			} else {
				/* we will make a path with one jog */
				path = new Point[4];
				path[0] = c0.getAbsoluteLocation();
				if (layout.verticalRows) {
					path[1] = new Point(
							cpos,c0.getAbsoluteY());
					path[2] = new Point(
							cpos,c1.getAbsoluteY());
				} else {
					path[1] = new Point(
							c0.getAbsoluteX(),cpos);
					path[2] = new Point(
							c1.getAbsoluteX(),cpos);
				}
				path[3] = c1.getAbsoluteLocation();
			}
			/* Set up the single path. */
			paths = new Point[1][];
			paths[0] = path;
		} else {
			/* more than two points. */
			paths = new Point[connectors.length+1][];
			if (layout.verticalRows) {
				/* draw the channel line */
				paths[0] = createPath(cpos,minR,cpos,maxR);
				/* Now draw a line from each connector to
				 * the channel line. */
				for (int c=0; c<connectors.length; c++) {
					Connector conn = connectors[c];
					paths[1+c] = createPath(
						conn.getAbsoluteX(),
						conn.getAbsoluteY(),
						cpos,conn.getAbsoluteY());
				}
			} else {
				/* draw the channel line */
				paths[0] = createPath(minR,cpos,maxR,cpos);
				/* Now draw a line from each connector to
				 * the channel line. */
				for (int c=0; c<connectors.length; c++) {
					Connector conn = connectors[c];
					paths[1+c] = createPath(
						conn.getAbsoluteX(),
						conn.getAbsoluteY(),
						conn.getAbsoluteX(),cpos);
				}
			}
		}
	}

	/** Create a path with two points. */
	protected Point[] createPath(int x0, int y0, int x1, int y1) {
		Point[] pp = new Point[2];
		pp[0] = new Point(x0,y0);
		pp[1] = new Point(x1,y1);
		return pp;
	}

	/** Sort an array of nets based on span. */
	protected static void sortBySpan(Net[] nets) {
		Sort.sortArray(nets,new SortNets());
	}

	/** Increase the bounds if necessary to include
	 * the paths of this net. */
	protected void addToBounds(BoundsRectangle bounds) {
		if (paths==null)
			return;		/* nothing to route on this net */
		for (int i=0; i<paths.length; i++) {
			Point[] path = paths[i];
			for (int j=0; j<path.length-1; j++) {
				Point p = path[j];
				bounds.add(p.x,p.y);
			}
		}
	}

	/** Draw all of the lines for this net. */
	protected void paint(LayoutGraphics g) {
		if (paths==null)
			return;	/* nothing to paint for this net */
		for (int i=0; i<paths.length; i++) {
			Point[] path = paths[i];
			for (int j=0; j<path.length-1; j++)
				g.drawLine(path[j],path[j+1]);
		}
	}

	/** Dump a net for debugging. */
	public void dump(PrintWriter f, String prefix, boolean details) {
		f.println(prefix+"Net: "+name);
		if (!details)
			return;
		f.println(prefix+"  rowNumber="+rowNumber+
					", set="+rowNumberSet);
		Connector.dump(f,connectors,prefix+" ",false);
	}

	/** Dump an array of nets. */
	public static void dump(PrintWriter f, Net[] nets, String prefix,
						boolean details) {
		f.print(prefix);
		f.print("Nets: ");
		if (nets==null) {
			f.println("null");
			return;
		}
		f.println(nets.length);
		for (int n=0; n<nets.length; n++) {
			nets[n].dump(f,prefix+" ["+n+"] ",details);
		}
	}
}

/* A class to sort nets based on span start. */
class SortNets implements SortComparator {
	/** Compare two nets, sort on R coordinate. */
	public int compare(Object n1, Object n2) {
		return ((Net)n1).minR - ((Net)n2).minR;
	}
}

/* end */
